/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Copyright 2010.  Los Alamos National Security, LLC. This material was    !
! produced under U.S. Government contract DE-AC52-06NA25396 for Los Alamos !
! National Laboratory (LANL), which is operated by Los Alamos National     !
! Security, LLC for the U.S. Department of Energy. The U.S. Government has !
! rights to use, reproduce, and distribute this software.  NEITHER THE     !
! GOVERNMENT NOR LOS ALAMOS NATIONAL SECURITY, LLC MAKES ANY WARRANTY,     !
! EXPRESS OR IMPLIED, OR ASSUMES ANY LIABILITY FOR THE USE OF THIS         !
! SOFTWARE.  If software is modified to produce derivative works, such     !
! modified software should be clearly marked, so as not to confuse it      !
! with the version available from LANL.                                    !
!                                                                          !
! Additionally, this program is free software; you can redistribute it     !
! and/or modify it under the terms of the GNU General Public License as    !
! published by the Free Software Foundation; version 2.0 of the License.   !
! Accordingly, this program is distributed in the hope that it will be     !
! useful, but WITHOUT ANY WARRANTY; without even the implied warranty of   !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General !
! Public License for more details.                                         !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#include "Matrix.h"

REAL M_TraceX2(Matrix A) {
  REAL trace=0.0;
  #ifdef CUDA
    REAL *device_trace;
    cudaMalloc(&device_trace, sizeof(REAL));
    // Calculate trace
    MatrixFastTraceX2Kernel<<<1,NUM_THREADS>>>(A.Device, device_trace, A.DM, NUM_THREADS);
    cudaThreadSynchronize();
    cudaMemcpy(&trace, device_trace, sizeof(REAL), cudaMemcpyDeviceToHost);
    cudaFree(device_trace);
  #endif
  #ifdef BLAS
    for (int i=0; i<A.M; i++)
      for (int j=0; j<A.M; j++) {
	trace+=A.Local[i+j*A.M]*A.Local[j+i*A.M];
      }
  #endif
  return trace;
}
